/*Class:offline_interpol

  This is the class for offline interpolation of the sparse grid

  You need to provide a surplus file in the following format

  dimension,  number of point,  total degree of freedom, 
  i_1..... i_dim  j_1 .... j_dim surplus_1 .... TotalDof

  Most member functions are self-explanatory.

*/ 


#ifndef offline_interpol_h_IS_INCLUDED
#define offline_interpol_h_IS_INCLUDED

#include "DataStructure.h"
#include <fstream>

class offline_interpol
{
public:

   offline_interpol();
   ~offline_interpol(){};

   virtual void LoadData(char* filename);

   virtual void Interpolate(Array<double>& x,double* value);

   virtual void Interpolate(double* x,double* value);

   virtual void Integrate(double* value);
   
   virtual void Integrate_local(Array<double>& x,double* value);
   
   virtual void Integrate_local(double* x,double* value);
   
   virtual double IndextoCoordinate(int i, int j);

   virtual double LinearBasis(double x, int i, int j);

   virtual double LinearBasisVolumeIntegral(int level);

   virtual void   PlotSparseGrid(char *filename = "grid.plt");

   //! \param Number of dimensions
   int dim ;
   //! \param Total Total number of points in grid
   int nno;
   //! \param Total degree of freedom
   int TotalDof;
   //! \param interpolation level
   int Level;

   //! Matrix to store the multi-index
   Matrix1<int> index;
   //! Matrix to store the surplus
   Matrix1<double> surplus;

};
#endif
