!=======================================================================  

!      subroutine box_to_cube_mapping(box_coord, cube_coord)
      
!=======================================================================         

      subroutine box_to_cube_mapping(box_coord, cube_coord,dim_sim, mu_up, &
                 mu_down, g_up, g_down, p_up, p_down)

      implicit none

      real(8), intent(in)  :: box_coord(dim_sim)
      real(8), intent(out) :: cube_coord(dim_sim)
      
      integer, intent(in) :: dim_sim
      real(8), intent(in) :: mu_up, mu_down, g_up, g_down, p_up, p_down
      
      real(8) :: scaling_mu, scaling_g, scaling_p 
      real(8), parameter :: range_cube = 1.0 !domain size [0..1]^d
   
      integer :: i
!-----------------------------------------------------------------------        
!.....check that dimensionality is a multiple of 2
      if (mod(dim_sim, 4) .ne. 0) then
        write(6,*) 'Error: dimensionality of simulation should be 4'
        stop
!-----------------------------------------------------------------------  
      else 
      
!-----------------------------------------------------------------------        
!.....linear transformation of mu, g, p    
        scaling_mu = (range_cube/(mu_up-mu_down))  !scaling for mu
        scaling_g  = (range_cube/(g_up- g_down))   !scaling for g               
        scaling_p  = (range_cube/(p_up-p_down))    !scaling for p   
      
!.....transformation to cube          
        cube_coord(1) = (box_coord(1) - mu_down)*scaling_mu  
        cube_coord(2) = (box_coord(2) - mu_down)*scaling_mu   
        cube_coord(3) = (box_coord(3) - g_down)*scaling_g  
        cube_coord(4) = (box_coord(4) - p_down)*scaling_p  
        
        do i = 1, dim_sim
          if (cube_coord(i) < 0.0) then 
            cube_coord(i) = 0.0
          else if (cube_coord(i) > 1.0) then
            cube_coord(i) = 1.0
          end if 
          
        end do
      
!-----------------------------------------------------------------------             
      end if
      
!-----------------------------------------------------------------------             
      end subroutine box_to_cube_mapping
!-----------------------------------------------------------------------         